# Generated file
# - do not overwrite
# - do not include in git
from collections import namedtuple

BuildInfo = namedtuple(
    'BuildInfo',
    ['version', 'built_at', 'sha', 'branch', 'tag'],
)

BUILD_INFO = BuildInfo(
    version='v4.10.5~64d4535',
    built_at='2024-09-13 10:11:53Z',
    sha='64d4535660de5e3668f42289574c581a8959364a',
    branch='HEAD',
    tag='v4.10.5',
)
